/* Automatically generated from date_object.cpp using ./create_hash_table. DO NOT EDIT ! */

#include "lookup.h"

namespace KJS {

const struct HashEntry dateTableEntries[] = {
   { "setMilliseconds", DateProtoFuncImp::SetMilliSeconds, DontEnum|Function, 1, 0 },
   { "setYear", DateProtoFuncImp::SetYear, DontEnum|Function, 1, 0 },
   { "toLocaleDateString", DateProtoFuncImp::ToLocaleDateString, DontEnum|Function, 0, &dateTableEntries[64] },
   { 0, 0, 0, 0, 0 },
   { "toString", DateProtoFuncImp::ToString, DontEnum|Function, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "setUTCSeconds", -DateProtoFuncImp::SetSeconds, DontEnum|Function, 2, 0 },
   { "setHours", DateProtoFuncImp::SetHours, DontEnum|Function, 4, 0 },
   { 0, 0, 0, 0, 0 },
   { "getTimezoneOffset", DateProtoFuncImp::GetTimezoneOffset, DontEnum|Function, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "setSeconds", DateProtoFuncImp::SetSeconds, DontEnum|Function, 2, 0 },
   { 0, 0, 0, 0, 0 },
   { "getUTCMinutes", -DateProtoFuncImp::GetMinutes, DontEnum|Function, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "getUTCFullYear", -DateProtoFuncImp::GetFullYear, DontEnum|Function, 0, 0 },
   { "toLocaleTimeString", DateProtoFuncImp::ToLocaleTimeString, DontEnum|Function, 0, 0 },
   { "toDateString", DateProtoFuncImp::ToDateString, DontEnum|Function, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "getUTCDate", -DateProtoFuncImp::GetDate, DontEnum|Function, 0, 0 },
   { "getMinutes", DateProtoFuncImp::GetMinutes, DontEnum|Function, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "getFullYear", DateProtoFuncImp::GetFullYear, DontEnum|Function, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "setUTCMinutes", -DateProtoFuncImp::SetMinutes, DontEnum|Function, 3, 0 },
   { 0, 0, 0, 0, 0 },
   { "setUTCFullYear", -DateProtoFuncImp::SetFullYear, DontEnum|Function, 3, 0 },
   { "getDate", DateProtoFuncImp::GetDate, DontEnum|Function, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "setUTCDate", -DateProtoFuncImp::SetDate, DontEnum|Function, 1, 0 },
   { "setMinutes", DateProtoFuncImp::SetMinutes, DontEnum|Function, 3, 0 },
   { "toTimeString", DateProtoFuncImp::ToTimeString, DontEnum|Function, 0, &dateTableEntries[61] },
   { "setFullYear", DateProtoFuncImp::SetFullYear, DontEnum|Function, 3, 0 },
   { 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "getUTCMilliseconds", -DateProtoFuncImp::GetMilliSeconds, DontEnum|Function, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "setDate", DateProtoFuncImp::SetDate, DontEnum|Function, 1, 0 },
   { 0, 0, 0, 0, 0 },
   { "getMonth", DateProtoFuncImp::GetMonth, DontEnum|Function, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "toLocaleString", DateProtoFuncImp::ToLocaleString, DontEnum|Function, 0, 0 },
   { "getTime", DateProtoFuncImp::GetTime, DontEnum|Function, 0, &dateTableEntries[63] },
   { "getUTCDay", -DateProtoFuncImp::GetDay, DontEnum|Function, 0, &dateTableEntries[65] },
   { "getYear", DateProtoFuncImp::GetYear, DontEnum|Function, 0, 0 },
   { "valueOf", DateProtoFuncImp::ValueOf, DontEnum|Function, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "toGMTString", -DateProtoFuncImp::ToGMTString, DontEnum|Function, 0, &dateTableEntries[66] },
   { 0, 0, 0, 0, 0 },
   { "getUTCSeconds", -DateProtoFuncImp::GetSeconds, DontEnum|Function, 0, 0 },
   { "getHours", DateProtoFuncImp::GetHours, DontEnum|Function, 0, 0 },
   { "toUTCString", -DateProtoFuncImp::ToUTCString, DontEnum|Function, 0, &dateTableEntries[62] },
   { 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0 },
   { "setTime", DateProtoFuncImp::SetTime, DontEnum|Function, 1, &dateTableEntries[67] },
   { "getUTCMonth", -DateProtoFuncImp::GetMonth, DontEnum|Function, 0, 0 },
   { "getDay", DateProtoFuncImp::GetDay, DontEnum|Function, 0, &dateTableEntries[68] },
   { "getUTCHours", -DateProtoFuncImp::GetHours, DontEnum|Function, 0, 0 },
   { "getSeconds", DateProtoFuncImp::GetSeconds, DontEnum|Function, 0, 0 },
   { "getMilliseconds", DateProtoFuncImp::GetMilliSeconds, DontEnum|Function, 0, &dateTableEntries[69] },
   { "setUTCMilliseconds", -DateProtoFuncImp::SetMilliSeconds, DontEnum|Function, 1, 0 },
   { "setUTCHours", -DateProtoFuncImp::SetHours, DontEnum|Function, 4, 0 },
   { "setMonth", DateProtoFuncImp::SetMonth, DontEnum|Function, 2, 0 },
   { "setUTCMonth", -DateProtoFuncImp::SetMonth, DontEnum|Function, 2, 0 }
};

const struct HashTable dateTable = { 2, 70, dateTableEntries, 61 };

} // namespace
